/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.etsi.saref.server;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.etsi.saref.server.entities.SAREFCore;
import org.etsi.saref.server.entities.SAREFExtension;
import org.etsi.saref.server.entities.SAREFProject;
import org.etsi.saref.server.entities.SAREFVersion;
import org.etsi.saref.server.entities.SAREFVersionName;

/**
*
* @author Maxime Lefrançois
*
*/
public class SAREF {

	public final static String BASE = "https://saref.etsi.org/";
	
	
	// project acronym
	public static final String REGEX_ACRONYM_VAR = "acronym";
	public static final String REGEX_ACRONYM = String.format("(?<%s>[a-z]{4})", REGEX_ACRONYM_VAR);

	// project name
	public static final String REGEX_NAME_EXTENSION = "saref4" + REGEX_ACRONYM;

	// project paths
	public static final String REGEX_PATH_CORE = "core";
	public static final String REGEX_PATH_EXTENSION = "saref4" + REGEX_ACRONYM;
	public static final String REGEX_PATH_PROJECT = String.format("(%s|%s)", REGEX_PATH_CORE, REGEX_PATH_EXTENSION);

	// project version
	public static final String REGEX_VERSION_MAJOR_VAR = "major";
	public static final String REGEX_VERSION_MINOR_VAR = "minor";
	public static final String REGEX_VERSION_PATCH_VAR = "patch";
	public static final String REGEX_VERSION_NUMBER = String.format("v(?<%s>[0-9]+)\\.(?<%s>[0-9]+)\\.(?<%s>[0-9]+)", REGEX_VERSION_MAJOR_VAR, REGEX_VERSION_MINOR_VAR, REGEX_VERSION_PATCH_VAR);

	// project version prefix, namespace, ontology IRI, ontology version IRI,
	public static final String REGEX_CORE_PREFIX = "saref";
	public static final String REGEX_EXTENSION_PREFIX = "s4" + REGEX_ACRONYM;

	public static final String REGEX_ONTOLOGY_IRI = String.format("^%s%s/$", BASE, REGEX_PATH_PROJECT);
	public static final String REGEX_ONTOLOGY_VERSION_IRI = String.format("^%s%s/%s/$", BASE, REGEX_PATH_PROJECT,
			REGEX_VERSION_NUMBER);

	// terms
	public static final String REGEX_TERM_NAME_VAR = "localName";
	public static final String REGEX_TERM_NAME = String.format("(?<%s>[^/]+)", REGEX_TERM_NAME_VAR);
	public static final String REGEX_TERM = String.format("^%s%s/%s$", BASE, REGEX_PATH_PROJECT, REGEX_TERM_NAME);
	
	

	public static SAREFProject extractProject(String uri) {
		final Matcher matcher = Pattern.compile(String.format("^%s%s/", BASE, REGEX_PATH_PROJECT))
				.matcher(uri);
		if (!matcher.find()) {
			return null;
		}
		final String acronym = matcher.group(REGEX_ACRONYM_VAR);
		if (acronym == null) {
			return SAREFCore.INSTANCE;
		} else {
			return new SAREFExtension(acronym);
		}
	}

	public static SAREFVersionName extractVersionName(String uri) {
		final Matcher matcher = Pattern.compile(String.format("^%s%s/%s/", BASE, REGEX_PATH_PROJECT,
				REGEX_VERSION_NUMBER)).matcher(uri);
		if (!matcher.find()) {
			return null;
		}
		int major = Integer.parseInt(matcher.group(REGEX_VERSION_MAJOR_VAR));
		int minor = Integer.parseInt(matcher.group(REGEX_VERSION_MINOR_VAR));
		int patch = Integer.parseInt(matcher.group(REGEX_VERSION_PATCH_VAR));
		return new SAREFVersionName(major, minor, patch);
	}

	public static SAREFVersion extractVersion(String uri) {
		final Matcher matcher = Pattern.compile(String.format("^%s%s/%s/", BASE, REGEX_PATH_PROJECT,
				REGEX_VERSION_NUMBER)).matcher(uri);
		if (!matcher.find()) {
			return null;
		}
		final SAREFProject project;
		final String acronym = matcher.group(REGEX_ACRONYM_VAR);
		if (acronym == null) {
			project= SAREFCore.INSTANCE;
		} else {
			project= new SAREFExtension(acronym);
		}

		int major = Integer.parseInt(matcher.group(REGEX_VERSION_MAJOR_VAR));
		int minor = Integer.parseInt(matcher.group(REGEX_VERSION_MINOR_VAR));
		int patch = Integer.parseInt(matcher.group(REGEX_VERSION_PATCH_VAR));
		final SAREFVersionName versionName = new SAREFVersionName(major, minor, patch);
		
		return new SAREFVersion(project, versionName);
	}
	
}
