/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.etsi.saref.servers.search;

import java.io.File;
import java.io.IOException;
import java.util.Collection;

import org.apache.jena.graph.Node;
import org.apache.jena.query.Dataset;
import org.apache.jena.query.text.EntityDefinition;
import org.apache.jena.query.text.TextDatasetFactory;
import org.apache.jena.query.text.TextIndex;
import org.apache.jena.query.text.TextIndexConfig;
import org.apache.jena.tdb.TDBFactory;
import org.apache.jena.vocabulary.RDFS;
import org.apache.lucene.analysis.en.EnglishAnalyzer;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.SimpleFSDirectory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DatasetManager {

	private static final Logger LOG = LoggerFactory.getLogger(DatasetManager.class);
	private static final String NAME_TDB = "tdb";
	private static final String NAME_LUCENE = "lucene";
	private static DatasetManager INSTANCE;
	private final File datasetDir;
	private final Directory dir;
	private final TextIndex textIndex;
	private final Dataset base;
	private final Dataset dataset;

	private DatasetManager() throws Exception {
		if (System.getenv("SAREF_PORTAL_DATASET") != null) {
			datasetDir = new File(System.getenv("SAREF_PORTAL_DATASET"));
		} else {
			datasetDir = new File("/home/saref-portal/dataset");
		}
		File tdbFile = new File(datasetDir, NAME_TDB);
		File luceneFile = new File(datasetDir, NAME_LUCENE);
		base = TDBFactory.createDataset(tdbFile.getAbsolutePath());
		EntityDefinition entDef = new EntityDefinition("uri", "text", "graph");
		entDef.setPrimaryPredicate(RDFS.label.asNode());
		entDef.setUidField("uid");
		entDef.setLangField("lang");
		Collection<Node> textPredicates = entDef.getPredicates("text");
		textPredicates.add(RDFS.comment.asNode());
		TextIndexConfig config = new TextIndexConfig(entDef);
		config.setAnalyzer(new EnglishAnalyzer());
		config.setValueStored(true);
		dir = new SimpleFSDirectory(luceneFile.toPath());
		textIndex = TextDatasetFactory.createLuceneIndex(dir, config);
		dataset = TextDatasetFactory.create(base, textIndex);
	}

	public static Dataset getDataset() {
		if (INSTANCE != null) {
			return INSTANCE.dataset;
		}
		try {
			INSTANCE = new DatasetManager();
			return INSTANCE.dataset;
		} catch (Exception ex) {
			LOG.error("Error while creating dataset", ex);
			return null;
		}
	}
	
	public static void close() {
		if (INSTANCE != null) {
			try {
				INSTANCE.dir.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			INSTANCE.base.close();
			INSTANCE.dataset.close();
//			INSTANCE.textIndex.close();
		}
	}

}
